%e_investigation
% Determination of e from the definition of the derivative
%
% ( f(x+h) - f(x) )/h  = f(x) if f(x) = e^x and h tends to 0.
%
% LAST UPDATED by Andy French April 2016

function e_investigation

%Define fontsize for graph
fsize = 16;

%Define function change
h = linspace(0.01,0.2,5);

%Define bases
b = linspace( 1, 5, 100);

%Construct mesh
[hh,b] = meshgrid(h,b);

%Define limit (for b = e this tends to 1 as dx tends to 0)
y = (b.^hh - 1)./hh;

%Plot limit vs base b for various h
figure('name','e investigation','color',[1 1 1])
plot(b,y);
hold on
xlabel('base b','fontsize',fsize)
ylabel('Limit (b^h - 1)/h','fontsize',fsize)
title(['Investigating e = ',num2str(exp(1)),...
    ' via consideration of limit (b^h - 1)/h'],'fontsize',fsize)
set(gca,'fontsize',fsize);
for n=1:5
    lgnd_str{n} = ['h = ',num2str(h(n))];
end
legend(lgnd_str,'location','bestoutside','fontsize',fsize)
grid on
xlimits = get(gca,'xlim');
ylimits = get(gca,'ylim');
plot( xlimits, [1,1],'k' );
plot( exp(1)*[1,1], ylimits,'k' );
print(gcf,'e_investigation','-dpng','-r300')
close(gcf);

%End of code