%euler
% Example of the use of a first order differential equation solver using
% the Euler method:
% dy_by_dx = f(x) 
% y(n+1) = y(n) + f(x)*deltax
% x(n+1) = x(x) + deltax
%
% LAST UPDATED by Andy French April 2016

function euler

%Define derivative dy_by_dx
dy_by_dx_eqa = 'x*exp(-y)';

%Define initial x,y coordinate
x0 = 0;
y0 = 1;

%Define number of iterations
N = 1000;

%Define finite change in x
dx = 0.1;

%Fontsize for graph
fsize = 16;

%

%Determine x,y coordinates
x = x0;
y = y0;
xx = x;
yy = y;
for n=1:N
    eval( [ 'dy_by_dx = ',F(dy_by_dx_eqa),';' ] );
    y = y + dy_by_dx*dx;
    x = x + dx;
    xx = [ xx, x];
    yy = [ yy, y];
end

%Create figure, plot x vs y and save a .png file
figure('name','euler','color',[1 1 1])
plot(xx,yy); hold on;
plot(x0,y0,'g*','markersize',fsize)
plot(x,y,'r*','markersize',fsize)
xlabel('x','fontsize',fsize)
ylabel('y','fontsize',fsize)
set(gca,'fontsize',fsize);
grid on
title(['dy/dx = ',dy_by_dx_eqa],'fontsize',fsize)
print( gcf, ['euler ',strrep(datestr(now),':','-')],'-dpng','-r300' )

%%

%F
% Function which adds . to enable elementwise matrix multiplication of
% variables in string s
function s = F(s)
s = strrep(s,'^','.^');
s = strrep(s,'*','.*');
s = strrep(s,'/','./');

%End of code