%newtonraphson
% Displays results of the Newton-Raphson root finding method.
%
% LAST UPDATED by Andy French April 2016

function newtonraphson

%Number of iterations
iter = 20;

%Number of graph evaluation points
N = 1000;

%Fontsize for graph
fsize = 16;

%

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Newton-Raphson root finder')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Define function to find x such that f(x) = 0
disp(' Define function to find x such that f(x) = 0 ')
f = input(' > f(x) = ','s'); 
if isempty(f)
    disp(' Default function is f(x) = 2*x^3+3*x^2-36*x+50');
    f = '2*x^3+3*x^2-36*x+50';
end
disp(' ')

%Define derivative of f(x)
fdash = input(' > fdash(x) = ','s'); 
if isempty(fdash)
    disp(' Default function is fdash(x) = 6*x^2+6*x-36' );
    fdash = '6*x^2+6*x-36';
end
disp(' ')

%Define initial guess for root
x1 = input(' > x1 = '); 
if isempty(x1)
    disp(' Default initial guess is x1 = -4' );
    x1 = -4;
end
disp(' ')

%Define graph limits
disp(' Define x limits in the form [xmin,xmax] ')
xlimits = input(' > [xmin,xmax] = '); 
if isempty(xlimits)
    disp(' Default xlimits are [-6,4]' );
    xlimits = [-8,5];
end
disp(' ')
disp(' Define y limits in the form [ymin,ymax] ')
ylimits = input(' > [ymin,ymax] = '); 
if isempty(ylimits)
    disp(' Default ylimits are [-100,100]' );
    ylimits = [-100,150];
end
disp( ' ' );

%

%Compute subsequent Newton-Raphson iterations and display to command window
disp( [' x1 = ',num2str(x1,'%15.12f')] );
x = x1;
for n=1:iter
    eval( ['x = x - (',f,')/(',fdash,');' ] );
    disp( [' x',num2str(n+1),'  = ',num2str(x,'%15.12f')] );
end
final_root = x;
disp(' ')

%Evaluate y vs x
x = linspace( xlimits(1),xlimits(2), N );
eval( ['y = ',F(f),';'] );

%Plot y vs x and save a .png file
figure('color',[1 1 1],'name','Newton Raphson')
plot(x,y);
xlabel('x','fontsize',fsize);
ylabel('y','fontsize',fsize);
title({ ['Newton-Raphson for y = ',f],...
    ['Root at ',num2str(final_root)] },'fontsize',fsize)
set(gca,'fontsize',fsize)
grid on
ylim(ylimits)
xlim(xlimits)
hold on;
plot( final_root, 0,'r*','markersize',fsize );
plot( xlimits, [0 0],'linewidth', 2,'color',[0,0,0] );
plot( [0,0], ylimits,'linewidth', 2,'color',[0,0,0] );
print( gcf, ['euler ',strrep(datestr(now),':','-')],'-dpng','-r300' )

%%

%F
% Function which adds . to enable elementwise matrix multiplication of
% variables in string s
function s = F(s)
s = strrep(s,'^','.^');
s = strrep(s,'*','.*');
s = strrep(s,'/','./');

%End of code