%mandeltraj
% Plots the first four iterations of a Mandelbrot complex transformation
% on the Argand diagram.
%
% LAST UPDATED by Andy French April 2016

function mandeltraj

% Define starting complex number
z0 = 0.5 - 0.3*1i;

%Define fontsize and marker size for plots
fsize = 14;
msize = 10;

%Define extent of Argand diagram
xmin = -2; xmax = 2; ymin = -2; ymax = 2;

%Define tick mark gaps
dx = 0.4;
dy = 0.4;

%

%Create Argand diagram
figure('color',[1 1 1],'name','mandlebrot trajectories');
axes( 'Nextplot', 'add');
plot( [0,0],[ymin,ymax],'k-' );
plot( [xmin,xmax],[0,0],'k-' );

%Store x and y values
x = [];
y = [];

%Plot initial z value
z = z0;
xx = real(z); yy = imag(z); x = [x,xx]; y = [y,yy];
plot( real(z), imag(z), 'k*','markersize', msize );

%Plot z after iteration 1
z = z^2 + z0;
xx = real(z); yy = imag(z); x = [x,xx]; y = [y,yy];
plot( real(z), imag(z), 'r*','markersize', msize );

%Plot z after iteration 2
z = z^2 + z0;
xx = real(z); yy = imag(z); x = [x,xx]; y = [y,yy];
plot( real(z), imag(z), 'g*','markersize', msize );

%Plot z after iteration 3
z = z^2 + z0;
xx = real(z); yy = imag(z); x = [x,xx]; y = [y,yy];
plot( real(z), imag(z), 'b*','markersize', msize );

%Plot z after iteration 4
z = z^2 + z0;
xx = real(z); yy = imag(z); x = [x,xx]; y = [y,yy];
plot( real(z), imag(z), 'm*','markersize', msize );

%Annotate axes
title(' First four iterations of Mandelbrot transformation','fontsize',fsize );
grid on
axis equal
box on
plot(x,y,'k')
xlim( [xmin,xmax] );
ylim( [ymin,ymax] );
set( gca, 'xtick', xmin: dx : xmax );
set( gca, 'ytick', ymin: dy : ymax );
set( gca, 'fontsize', fsize );
xlabel( 'Re(z)','fontsize',fsize )
ylabel( 'Im(z)','fontsize',fsize )
print(gcf,'Mandelbrot iterations.png','-dpng','-r300')

%End of code