%argand
% Plots a complex number (and a few powers of it) on the Argand diagram
%
% LAST UPDATED by Andy French April 2016

function argand

%Display message to command window
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Argand diagram plotter')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Define polar angle arg(z) and range |z| of complex number z
disp(' Define polar angle arg(z) (radians) and range |z| of complex number z' )
theta = input(' > arg(z) in radians = ');
if isempty(theta)
    theta = pi/5;
    disp(' arg(z) = pi/5 is the default option ')
end
r = input(' > r = ');
if isempty(r)
    r = sqrt(2);
    disp(' r = sqrt(2) is the default option ')
end
disp(' ');

%

%Determine complex number z
z = r*(cos(theta) + 1i*sin(theta) );

%Plot z, z^2 and z^3 on an Argand Diagram
figure( 'name', 'Argand diagram', 'color', [1 1 1] );
plot( real(z), imag(z), 'r*','markersize', 18 );
hold on
plot( [0,real(z)],[0,imag(z)],'r-' );
plot( real(z^2), imag(z^2), 'g*','markersize', 18 );
plot( [0,real(z^2)],[0,imag(z^2)],'g-' );
plot( real(z^3), imag(z^3), 'b*','markersize', 18 );
plot( [0,real(z^3)],[0,imag(z^3)],'b-' );
grid on
axis equal
set(gca, 'fontsize', 18)
title_str = [' z = x + iy = ',num2str(r),...
    '( cos(',num2str(theta),') + isin(',num2str(theta),') )'];
xlabel( 'x = Re(z)','fontsize',18 )
ylabel( 'y = Im(z)','fontsize',18  )
title( title_str ,'fontsize',18 )

%Print .png file of graph
print( gcf, [title_str,'.png'],'-dpng','-r300' );

%End of code