%circle_tangent
%
% Function which computes tangents to a circle of radius R, centre (a,b)
% at polar angles t1 and t2 (degrees anticlockwise from the horizontal).
% The intersection of these tangents is computed.
% The circle, plus the tangents, are then plotted.
%
% LAST UPDATED by Andy French December 2012

function circle_tangent

%% INPUTS %%

%Polar angles of point of intersection of circle at which tangent is
%computed
t1 = (180/pi)*atan(4/3);
t2 = 90 + (180/pi)*atan(4/3);

%Radius of circle
R = 5;

%x and y coordinates of circle centre
a = 1;
b = 2;

%%

%Convert angles to radians
t1 = pi*t1/180;
t2 = pi*t2/180;

%Compute gradient (m) and y intercepts of tangents
[m1,c1] = ct(t1,R,a,b);
[m2,c2] = ct(t2,R,a,b);

%Work out intersection cordinates of tangents
xi = (c2-c1)/(m1-m2);
yi = m1*xi + c1;
ycheck = yi - (m2*xi + c2);

%Determine min and max x and y values to fit in all the interesting
%features
if isnan(xi)==1
    xmin = min([ 0,a-R ]);
    xmax = max([ 0,a+R ]);
    ymin = min([ 0,b-R ]);
    ymax = max([ 0,b+R ]);
else
    xmin = min([ 0,a-R, xi ]);
    xmax = max([ 0,a+R, xi ]);
    ymin = min([ 0,b-R, yi ]);
    ymax = max([ 0,b+R, yi ]);
end
xmin = 1.1*xmin;
xmax = 1.1*xmax;
ymin = 1.1*ymin;
ymax = 1.1*ymax;

%Create data vectors for plotting
N = 100;
t = linspace(0,2*pi,N);
x = linspace( xmin, xmax, N);
y1 = m1*x + c1;
y2 = m2*x + c2;
xc = R*cos(t) + a;
yc = R*sin(t) + b;

%Plot scene
figure('name','Circle Tangent','color', [1 1 1])
axis equal
hold on;
plot(x,y1,'r');
plot(x,y2,'g');
legend({ ['y1 = ',num2str(m1),'x + ',num2str(c1)],...
    ['y2 = ',num2str(m2),'x + ',num2str(c2)] },'location','Eastoutside');
plot(xc,yc,'b');
plot(xi,yi,'m*')
plot( R*cos(t1)+a, R*sin(t1)+b,'r*');
plot( R*cos(t2)+a, R*sin(t2)+b,'g*');
plot( a, b, 'bx')
plot( [a, R*cos(t1)+a],[b, R*sin(t1)+b], 'r--' );
plot( [a, R*cos(t2)+a],[b, R*sin(t2)+b], 'g--' );
grid on;
xlabel('x')
ylabel('y')
title({
    'Circle Tangent',...
    ['Radius = ',num2str(R),', a = ',num2str(a),', b = ',num2str(b)],...
    ['t1 = ',num2str(t1*180/pi),'^o, t2 = ',num2str(t2*180/pi),'^o'],...
    ['xi = ',num2str(xi),', yi = ',num2str(yi)],...
    },'HorizontalAlignment','left');
xlim([xmin,xmax]);
ylim([ymin,ymax]);
% xlims = get(gca,'xlim');
% ylims = get(gca,'ylim');
% xmin = xlims(1);
% xmax = xlims(2);
% ymin = ylims(1);
% ymax = ylims(2);
plot([0,0],[ymin,ymax],'k','linewidth',1)
plot([xmin,xmax],[0,0],'k','linewidth',1)

%Save PNG output
print(gcf,['circle tangent ',strrep(datestr(now),':','-')],'-dpng','-r300');
close(gcf);

%%

%Circle tangent function. Works out tangent in the form y = mx + c
function [m,c] = ct(t,R,a,b)
m = -1/tan(t);
c = R*sin(t) + b + (R*cos(t) + a)/tan(t);

%End of code