%regpoly
% Plots a regular polygon of N sides within a circle. An inner circle is
% also plotted.
%
% LAST UPDATED by Andy French 02/02/2012

function regpoly

%Define max number of sides of polygon
Nmax = 20;

%Define radius
r = 1;

%Define number of points to evaluate ... vs N plots
numN = 1000;

%

for N=3:Nmax
    %Compute parameters for polygon N
    [eang_deg,iang_deg,tot_iang_deg,a,A,P,R,dAo,dAi] = regpolyparams(N,r);
    A_over_pi_r2 = A/(pi*r^2); disp(['Area = pi*r^2 * ',num2str(A_over_pi_r2)]);
    P_over_2pi_r =  P/(2*pi*r); disp(['Polygon perimeter = pi*r^2 * ',num2str(P_over_2pi_r)]);
    dAo_over_pi_r2 = dAo/(pi*r^2); disp(['Outer circle area - polygon area = pi*r^2 * ',num2str(dAo_over_pi_r2)]);
    dAi_over_pi_r2 =  dAi/(pi*r^2); disp(['Polygon aea - inner circle area = pi*r^2 * ',num2str(dAi_over_pi_r2)]);
    
    %Plot polygon
    plotregpoly(N,r);
    saveas(gcf,['regpoly_circle N=',leadingzero(N,3)],'png')
    close(gcf);
end

%Work out vectors of paramters for shapes for N....Nmax, but use fractional
%N to make the graphs smooth
N = linspace(3,Nmax,numN);
[eang_deg,iang_deg,tot_iang_deg,a,A,P,R,dAo,dAi] = regpolyparams(N,r*ones(1,numN));
A_over_pi_r2 = A/(pi*r^2);
P_over_2pi_r =  P/(2*pi*r);
dAo_over_pi_r2 = dAo/(pi*r^2);
dAi_over_pi_r2 =  dAi/(pi*r^2);

%Plot results
figure('color',[1 1 1]);
plot( N, A_over_pi_r2, N, P_over_2pi_r,...
    N, dAo_over_pi_r2, N, dAi_over_pi_r2 )
xlabel('Number of sides N')
legend({'Polygon area/\pir^2','Polygon perimeter/2\pir',...
    '(Outer circle - polygon area) /\pir^2','(Polygon - inner circle area)/\pir^2'},'location','SouthOutside')
xlim([3,Nmax])
title('Polygon perimeter and area ratios vs N')
saveas(gcf,'repoly_ratios','png')
close(gcf);

%%

%Compute polygon parameters for vector inputs of radius r and number of
%sides N
function [eang_deg,iang_deg,tot_iang_deg,a,A,P,R,dAo,dAi] = regpolyparams(N,r)

%Compute exterior angle t /radians
eang_deg = 360./N;
t = eang_deg*pi/180;

%Compute internal angle /degrees and total internal angle /degrees
iang_deg = 180 - eang_deg;
tot_iang_deg = N.*iang_deg ;

%Compute side length / radius
a = 2*r.*sin( 0.5*t );

%Compute area
A = 0.5*N.*(r.^2).*sin(t);

%Compute perimeter
P = N.*a ;

%Compute radius of inner circle
R = r.*cos(0.5*t);

%Work out difference between outer circle and inner polygon areas
dAo = pi*r.^2 - A;

%Work out difference between polygon and inner circle areas
dAi = A - pi*R.^2;

%%

%Plot a polygon of radius r and number of sides N in addition to an inner
%and outer circle touching the polygon
function plotregpoly(N,r)

%Get polygon parameters
[eang_deg,iang_deg,tot_iang_deg,a,A,P,R] = regpolyparams(N,r);

%Define angle vector to enable the circles to be drawn
theta = linspace(0,2*pi,1000);

%Outer circle
xo = r*sin(theta);
yo = r*cos(theta);

%Inner circle
xi = R*sin(theta);
yi = R*cos(theta);

%Create polygon vertices
x = 0;
y = r;
for n=1:N
    x = [x,r*sin(2*pi*n/N)];
    y = [y,r*cos(2*pi*n/N)];
end
x = [x,0];
y = [y,r];

%Plot polygon
fig_title = ['Regular polygon of ',num2str(N),' sides, radius = ',num2str(r),'. Side length = ',num2str(a)]
figure('name',fig_title,'color',[1 1 1]);
patch(xo,yo,[1,1,0]);
hold on;
patch(x,y,'r');
patch(xi,yi,'w');
axis equal
grid on
box on
alpha(0.3)
fig_title = {fig_title,...
    ['Internal angle = ',num2str(iang_deg),'^o, area = ',num2str(A),', inner circle radius = ',num2str(R)] };
fig_title = char(fig_title);
title(fig_title)

%%

%Indexed images of name 00001, 00002 etc
function outstring = leadingzero(X,N)
outstring = [strrep(blanks(N-1 - fix(log10(X))),...
    ' ','0'),num2str(X,3)];

%End of code