% Function which generates and plots a triangle for demonstration of sine
% and cosine rules, especially problems which may arise from use of the
% sine rule when one angle is obtuse (i.e. greater than 90 degrees). Note
% application of the cosine rule will ALWAYS work.This is because sine is
% multi-valued in the range 0-180 degrees but cosine is NOT.
%
% LAST UPDATED by Andy French May 2016

function triangle

%Display answers in command window?
display_ans = 1;

%Font size
fsize = 18;

%%

%Ask user for two lengths and the angle between these lengths
disp(' ');
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%' )
disp(' %%% Welcome to TRIANGLE by Andy French %%%%')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%' )
disp(' ')

%Get inputs from user
c = -1;
b = -1;
A = 190;
while c<=0
    c = str2num( input( ' ... Input length of side c joining vertices A to B \n','s') );
    if c<0
        disp('Naughty! c<=0. Try again.')
    elseif isempty(c)
        c = 3;
        disp(' Default c = 3 set')
        disp(' ')
    end
end
while b<=0
    b = str2num( input( ' ... Input length of side b joining vertices A to C \n','s') );
    if b<=0
        disp('Naughty! b<=0. Try again.')
    elseif isempty(b)
        b = 5;
        disp(' Default b = 5 set')
        disp(' ')
    end
end
while A==0 || abs(A)>=180
    A = str2num( input( ' ... Input angle A between sides c and b /degrees \n','s') );
    if isempty(A)
        A = 45;
        disp(' Default A = 45 degrees set')
    elseif (A==0) || (abs(A)>=180)
        disp('Naughty! A = 0 or A >= 180. Try again.')          
    end
end
disp(' ')

%Compute other side via the Cosine Rule
a = sqrt( c^2 + b^2 - 2*b*c*cos(A*pi/180) );
if display_ans == 1;
    disp( [' ... Side a = ',num2str(a) ] );
end

%Compute other two angles via the Cosine Rule
C = (180/pi)*acos( (b^2 + a^2 - c^2 )/( 2*a*b ) );
B = (180/pi)*acos( (c^2 + a^2 - b^2 )/( 2*a*c ) );
if display_ans == 1;
    disp( [' ... Angle C = ',num2str(C),' degrees' ] );
    disp( [' ... Angle B = ',num2str(B),' degrees' ] );
end

%Check angles all sum to 180 degrees
disp( [' ... Angles sum to A + B + C = ',num2str(A + B + C),' degrees'] )
disp(' ')

%Plot triangle, setting centre of triangle to be at the origin ....

%Solved triangle
figure('color',[1 1 1],'name','triangle solved','units','normalized')
set(gcf,'position',[0 0 1 1])
x = [ 0, a, a-b*cos(C*pi/180)];
y = [ 0, 0, b*sin(C*pi/180) ];
xL = [ a/2,  a-0.5*b*cos(C*pi/180), 0.5*c*cos(B*pi/180) ];
yL = [ 0,  0.5*b*sin(C*pi/180), 0.5*c*sin(B*pi/180) ];
xm = mean(x);
ym = mean(y);
patch(x-xm,y-ym,'g','facealpha',0.5,'edgecolor','none')
hold on

%Label angles and sides
text( x-xm,y-ym,{'B','C','A'},'fontsize',fsize,'fontweight','bold','edgecolor',[ 0 0 0 ] )
text( xL-xm,yL-ym,{'a','b','c'},'fontsize',fsize,'fontweight','bold','edgecolor',[ 1 0 0 ] )

%Set axis limits and scaling
axis equal
xlim([ floor(1.1*min(x-xm)),ceil(1.1*max(x-xm)) ])
ylim([ floor(1.1*min(y-ym)),ceil(1.1*max(y-ym)) ])
grid on
box on
set(gca,'fontsize',fsize)

%Define title containing all of the parameters of the triangle
title( char( {['Triangle: a = ',num2str(a,'%5.2f'),...
    ', b = ',num2str(b,'%5.2f'),', c = ',num2str(c,'%5.2f') ] ;...
    ['A = ',num2str(A,'%5.2f'),...
    '^o, B = ',num2str(B,'%5.2f'),'^o, C = ',num2str(C,'%5.2f'),'^o' ]}),'fontsize',fsize );

%Save PNG output
saveas(gcf, 'solved_triangle', 'png');

%

%Unsolved triangle
figure('color',[1 1 1],'name','triangle unsolved','units','normalized')
set(gcf,'position',[0 0 1 1])
x = [ 0, a, a-b*cos(C*pi/180)];
y = [ 0, 0, b*sin(C*pi/180) ];
xL = [ a/2,  a-0.5*b*cos(C*pi/180), 0.5*c*cos(B*pi/180) ];
yL = [ 0,  0.5*b*sin(C*pi/180), 0.5*c*sin(B*pi/180) ];
xm = mean(x);
ym = mean(y);
patch(x-xm,y-ym,'g','facealpha',0.5,'edgecolor','none')
hold on

%Label angles and sides
text( x-xm,y-ym,{'B','C','A'},'fontsize',fsize,'fontweight','bold','edgecolor',[ 0 0 0 ] )
text( xL-xm,yL-ym,{'a','b','c'},'fontsize',fsize,'fontweight','bold','edgecolor',[ 1 0 0 ] )

%Set axis limits and scaling
axis equal
xlim([ floor(1.1*min(x-xm)),ceil(1.1*max(x-xm)) ])
ylim([ floor(1.1*min(y-ym)),ceil(1.1*max(y-ym)) ])
grid on
box on
alpha(0.6)
set(gca,'fontsize',fsize)

%Define title containing all of the parameters of the triangle
title( char( {['Triangle: a = ?',...
    ', b = ',num2str(b,'%5.2f'),', c = ',num2str(c,'%5.2f') ] ;...
    ['A = ',num2str(A,'%5.2f'),...
    '^o, B = ?, C = ?' ]}),'fontsize',fsize );

%Save PNG output
saveas(gcf, 'unsolved_triangle', 'png');

%End of code