% Plot vector equations of straight lines
%
% LAST UPDATED by A. French May 2014

function vec_eqa_line

%Define range of scalar parameters s and t
smin = -10;
smax = 10;
tmin = -4;
tmax = 5;

%%

% Line 1: r1 = a1 + s*d1

%Point on line
a1 = [
    1
    4
    2
    ];

%Direction vector
d1 = [
    -3
    -1
    1
    ];

% Line 2: r2 = a2 + t*d2

%Point on line
a2 = [
    1
    0
    2
    ];

%Direction vector
d2 = [
    3
    -1
    -1
    ];

%Fontsize
fsize = 20;

%Number of line points
N = 100;

%%

%Determie if lines are askew. (D is the minimum distance between the two
%lines)
D = abs( dot( a2 - a1, cross(d1,d2) ) ) /...
    sqrt( dot( cross(d1,d2),cross(d1,d2) ) );

%If lines are not askew, determine coordinates of intersection
if D==0
mu = dot( cross( a1 - a2, d1 ), cross( d2,d1 ) )/...
    dot(cross( d2,d1 ),cross( d2,d1 ));
I = a2 + mu*d2;
else
I = [];
end

%Determine vector of line coordinates
s = linspace(smin,smax,N);
t = linspace(tmin,tmax,N);
x1 = a1(1) + d1(1)*s ;
y1 = a1(2) + d1(2)*s ;
z1 = a1(3) + d1(3)*s ;
x2 = a2(1) + d2(1)*t ;
y2 = a2(2) + d2(2)*t ;
z2 = a2(3) + d2(3)*t ;

%Plot lines and axes
set(gcf,'name','vector equation of a line',...
    'color',[1 1 1],'units','pixels');
clf;
plot3(x1,y1,z1,'r','linewidth',4);
hold on
plot3(x2,y2,z2,'b','linewidth',4);
legend({'line 1','line 2'})
xlimits = get(gca,'xlim');
ylimits = get(gca,'ylim');
zlimits = get(gca,'zlim');
text(xlimits(2)*1.05,0,0,'x','fontsize',fsize)
text(0,ylimits(2)*1.05,0,'y','fontsize',fsize)
text(0,0,zlimits(2)*1.05,'z','fontsize',fsize)
grid on
box on
axis vis3d
set(gca,'fontsize',fsize);
plot3(xlimits,[0,0],[0,0],'k','linewidth',4);
plot3([0,0],ylimits,[0,0],'k','linewidth',4);
plot3([0,0],[0,0],zlimits,'k','linewidth',4);
plot3(I(1),I(2),I(3),'m*','markersize',fsize);
plot3(a1(1),a1(2),a1(3),'r*','markersize',fsize);
plot3(a2(1),a2(2),a2(3),'b*','markersize',fsize);
view([114,32])
if ~isempty(I)
    title(['Line intersection at (',num2str(I(1)),',',...
        num2str(I(2)),',',num2str(I(3)),')'],'fontsize',fsize)
else
    title(['Lines askew. Minimum separation = ',num2str(D)]);
end
print(gcf,'vec_eqa_line.png','-dpng','-r300');

%End of code