%cubic
% Generates one of three types of 'nice' cubics and plots the curve.
%
% Option 1: Three real roots
% Option 2: A single real root
% Option 3: A cubic defined by its stationary points
%
% LAST UPDATED by Andy French April 2016

function cubic

%Fontsize for graph
fsize = 18;

%

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Generate a nice cubic equation ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Get cubic option from user
disp(' Choose cubic generating option. ')
disp(' ')
disp(' Option 1: Three real roots ')
disp(' Option 2: A single real root ')
disp(' Option 3: A cubic defined by its stationary points ')
disp(' ')
option = input(' > option = ');
if isempty(option)==1
    option = 3;
    disp(' Option 3 is the default choice.')
end

%Get parameters from user
if option ==1
    
    % Option 1: Three real roots
    disp(' ');
    disp(' Define real numbers a,b,c which specify cubic y = x^3 -Ax^2 +Bx - C')
    disp(' in factorized form y = ( x - a )( x - b  )(x - c ) ');
    disp(' ');
    
    %Set up an display a dialogue box
    prompt = {'a','b','c'};
    dlg_title = 'y = ( x - a )( x - b  )(x - c ) ';
    num_lines = 1;
    r = randi([-9,9],3);
    for n=1:3
        def{n} = num2str(r(n));
    end;
    answer = inputdlg(prompt,dlg_title,num_lines,def);
    if isempty(answer)
        return
    end
    
    %Determine parameters a,b,c
    a = str2num( answer{1} );
    b = str2num( answer{2} );
    c = str2num( answer{3} );
    
    %Determine A,B,C
    A = a + b + c;
    B = a*b + a*c + b*c;
    C = a*b*c;
    D = 1;
    
    %Determine roots of the cubic (check - these should be a,b,c)
    cubic_roots = roots([D,-A,B,-C]).';
    
    %Generate a vector of x values
    x = linspace( 1.5*min([cubic_roots,0]),1.5*max([cubic_roots,0]),1000);
    
elseif option ==2
    
    % Option 2: A single real root
    disp(' ');
    disp(' Define real numbers a,b,c which specify cubic y = x^3 -Ax^2 +Bx - C')
    disp(' in factorized form y = ( x - a )( x - b - ic )(x - b + ic ) ');
    disp(' ');
    
    %Set up an display a dialogue box
    prompt = {'a','b','c'};
    dlg_title = 'y = ( x - a )( x - b - ic )(x - b + ic ) ';
    num_lines = 1;
    r = randi([-9,9],3);
    for n=1:3
        def{n} = num2str(r(n));
    end;
    answer = inputdlg(prompt,dlg_title,num_lines,def);
    if isempty(answer)
        return
    end
    
    %Determine parameters a,b,c
    a = str2num( answer{1} );
    b = str2num( answer{2} );
    c = str2num( answer{3} );
    
    %Determine A,B,C,D
    A = a + 2*b;
    B = b^2 + c^2 + 2*a*b;
    C = a*(b^2 + c^2);
    D = 1;
    
    %Determine x values of stationary points
    xs1 = (2*A - sqrt(4*A^2 -12*D*B) )/(6*D);
    xs2 = (2*A + sqrt(4*A^2 -12*D*B) )/(6*D);
    
    %Determine roots of the cubic
    cubic_roots = a;
    
    %Generate a vector of x values. Use stationary points, unless they are
    %imaginary
    if abs( imag(xs1) ) >0
        x = linspace( 1.5*min([a,-a]),...
            1.5*max([a,-a]),1000);
    else
        x = linspace( 1.5*min([a,-a,xs1,xs2,-xs1,-xs2]),...
            1.5*max([a,-a,xs1,xs2,-xs1,-xs2]),1000);
    end    
else
    
    % Option 3: A cubic defined by its stationary points
    
    %Set up an display a dialogue box
    prompt = {'Stationary point 1 x:','Stationary point 1 y:',...
        'Stationary point 2 x:','Stationary point 2 y:'};
    dlg_title = 'Define stationary points of a cubic';
    num_lines = 1;
    r = randi([-5,5],4);
    for n=1:4
        def{n} = num2str(r(n));
    end;
    answer = inputdlg(prompt,dlg_title,num_lines,def);
    if isempty(answer)
        return
    end
    
    %Determine parameters a,b,c,d
    a = str2num( answer{1} );
    b = str2num( answer{2} );
    c = str2num( answer{3} );
    d = str2num( answer{4} );
    
    %Determine parameters of the cubic y = D*x^3 -A*x^2 + B*x - C
    k = (b-d)/( (1/3)*(a^3 - c^3) + (1/2)*(a+c)*(c^2 - a^2) + a*c*(a-c) );
    e = b - (1/3)*k*a^3 + (1/2)*k*(a+c)*a^2 - a*c*a*k;
    ee = d - (1/3)*k*c^3 + (1/2)*k*(a+c)*c^2 - a*c*c*k;
    check_is_zero = e - ee;
    D = (1/3)*k;
    A = (1/2)*k*(a+c);
    B = a*c*k;
    C = -e;
    
    %Determine x values of stationary points
    xs1 = (2*A - sqrt(4*A^2 -12*D*B) )/(6*D);
    xs2 = (2*A + sqrt(4*A^2 -12*D*B) )/(6*D);
    
    %Determine roots of the cubic, and ignore imaginary ones
    cubic_roots = roots([D,-A,B,-C]).';
    cubic_roots( imag(cubic_roots)~=0 ) =[];
    
    %Generate a vector of x values
    xmin = 1.5*min([a,c,cubic_roots,-a,-c,-cubic_roots]);
    xmax = 1.5*max([a,c,cubic_roots,-a,-c,-cubic_roots]);
    x = linspace( xmin,xmax,100 );
end

%Generate cubic
y = D*x.^3 -A*(x.^2) +B*x - C;

%Construct title string to avoid + - issues!
if A>0
    Astr = ['- ',num2str(A)];
else
    Astr = ['+ ',num2str(-A)];
end
if B>0
    Bstr = ['+ ',num2str(B)];
else
    Bstr = ['- ',num2str(-B)];
end
if C>0
    Cstr = ['- ',num2str(C)];
else
    Cstr = ['+ ',num2str(-C)];
end
if D==1
    title_str = [ 'y = x^3 ',Astr,'x^2 ',Bstr,'x ',Cstr];
else
    title_str = [ 'y = ',num2str(D),'x^3 ',Astr,'x^2 ',Bstr,'x ',Cstr];
end

%Determine .png filename
filename = [title_str,'.png'];

%Plot cubic
figure('name','cubic','color',[1 1 1])
plot(x,y);
xlabel('x','fontsize',fsize)
ylabel('y','fontsize',fsize)
set(gca, 'fontsize',fsize)
grid on
hold on

%Plot annotation, germane to option chosen
if option==1
    %Plot roots
    plot( a, 0, 'r*','markersize',fsize )
    plot( b, 0, 'g*','markersize',fsize )
    plot( c, 0, 'b*','markersize',fsize )
    
    %Make title string - remember it is y = (x-a)(x-b)(x-c) for this option
    if a<0
        asign = ' + ';
    else
        asign = ' - ';
    end
    if b<0
        bsign = ' + ';
    else
        bsign = ' - ';
    end
    if c<0
        csign = ' + ';
    else
        csign = ' - ';
    end
    title_str = {title_str,...
        ['y = ( x',asign,num2str(abs(a)),...
        ' )( x',bsign,num2str(abs(b)),' )( x',...
        csign,num2str(abs(c)),' )'] };
elseif option==2
    plot( a, 0, 'r*','markersize',18 )
    title_str = {title_str,[' Single root is x = ',num2str(a)] };
else
    if length(cubic_roots) == 1
        %Single real root. The other two are a conjugate pair of complex
        %numbers i.e. a+ib, a-ib.
        title_str = {title_str,...
        ['y intercept = (0,',num2str(e),'), root is: ',num2str( cubic_roots(1) )],...
        ['Stationary points are: (',answer{1},...
        ',',answer{2},') and (',answer{3},',',answer{4},')'] };
    else
        title_str = {title_str,...
        ['y intercept = (0,',num2str(e),'), roots are: ',num2str( cubic_roots(1) ),...
        ', ',num2str( cubic_roots(2) ),', ',num2str( cubic_roots(3) ),],...
        ['Stationary points are: (',answer{1},...
        ',',answer{2},') and (',answer{3},',',answer{4},')'] };
    end    
    plot(a,b,'r*','markersize',fsize);
    plot(c,d,'b*','markersize',fsize);
    plot( cubic_roots, [0,0,0],'go','markersize',fsize )
end
title( title_str,'fontsize',fsize );
axis tight

%Plot x and y axes
xlimits = get(gca,'xlim');
ylimits = get(gca,'ylim');
ymin = -min(abs(ylimits));
ymax = -ymin;
ylimits = [ymin,ymax];
set(gca,'ylim',ylimits);
plot( [0,0], ylimits, 'k-' );
plot( xlimits,[0,0], 'k-' );

%Print a .png file
print( gcf,filename, '-dpng','-r300');

%Display final message
disp(' Cubic plots generated.')
disp(' ')

% End of code

