%parametric
% Function which plots Cartesian (x,y) curve defined parametrically.
%
% LAST UPDATED by Andy French 20/1/2016

function parametric

%Set anotation size
fsize = 16;

%Number of x,y curve evaluation points
N = 1000;

%Number of x,y curve specific values indicated by a black star
NN = 10;

%

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp('  Plot curves defined parametrically ')
disp('  x = f(t),  y = g(t) ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp(' ')

%Determine inputs from user

%Define parametric equations in terms of parameer t
disp(' Define parametric equations:  x = f(t), y = g(t) ')
x_eqa= input(' x = f(t) =  ','s');
if isempty(x_eqa)
    disp(' x = exp(t) + 1 chosen by default.')
    x_eqa = 'exp(t) + 1';
end
disp(' ')
y_eqa= input(' y = g(t) =  ','s');
if isempty(y_eqa)
    disp(' y = exp(t) + t chosen by default.')
    y_eqa = 'exp(t) + t';
end
disp(' ')

%Define derivatives of parametric equations in terms of parameer t
disp(' Define parametric equation derivatives')
dx_by_dt_eqa= input(' dx_by_dt =  ','s');
if isempty(dx_by_dt_eqa)
    disp(' dx_by_dt = exp(t) chosen by default.')
    dx_by_dt_eqa = 'exp(t)';
end
disp(' ')
dy_by_dt_eqa= input(' dy_by_dt =  ','s');
if isempty(dy_by_dt_eqa)
    disp(' dy_by_dt = exp(t) + 1 chosen by default.')
    dy_by_dt_eqa = 'exp(t) + 1';
end
disp(' ')

%Define range of parameter t
disp(' Define range of parameter t ')
tmin= input( ' tmin = ');
if isempty(tmin)
    disp(' tmin = 0 chosen by default.')
    tmin = -5;
end
tmax= input( ' tmax = ');
if isempty(tmax)
    disp(' tmax = 12*pi chosen by default.')
    tmax = 3;
end
disp(' ')

%Set value of t to compute tangent and normal
disp(' Define t to compute tangent and normal to parametric curve ')
tt= input( ' t = ');
if isempty(tt)
    disp(' t = 0 chosen by default.')
    tt = 0;
end

%

% Work out vectors for t,x,y
t = linspace(tmin,tmax,N);
eval( ['x = ',F(x_eqa),';'] );
eval( ['y = ',F(y_eqa),';'] );

% Work out vectors for t,x,y evaluated at specific points
ttt = linspace(tmin,tmax,NN);
eval( ['xxx = ',strrep( F(x_eqa),'t','ttt' ),';'] );
eval( ['yyy = ',strrep( F(y_eqa),'t','ttt' ),';'] );

%Work out graph limits
xmin = min( [min(x),0] );
xmax = max( [max(x),0] );
ymin = min( [min(y),0] );
ymax = max( [max(y),0] );

%Find the gradient at t = tt
eval( ['dx_by_dt = ',strrep(F(dx_by_dt_eqa),'t','tt'),';'] );
eval( ['dy_by_dt = ',strrep(F(dy_by_dt_eqa),'t','tt'),';'] );
dy_by_dx = dy_by_dt/dx_by_dt;

%Find x,y at t = tt and hence find equations of normal and tangent
eval( ['xx = ',strrep(F(x_eqa),'t','tt'),';'] );
eval( ['yy = ',strrep(F(y_eqa),'t','tt'),';'] );
X = linspace( xmin,xmax,N);
yt =  dy_by_dx*(X-xx) + yy;
yn =  -(1/dy_by_dx)*(X-xx) + yy;
yt_str = ['y_T = ',num2str(dy_by_dx), 'x + ',num2str( yy - dy_by_dx*xx ) ];
yn_str = ['y_N = ',num2str(-1/dy_by_dx), 'x + ',num2str( yy + xx/dy_by_dt) ];

%Plot parametric curve and overlay tangent and normal
figure('color',[1 1 1],'name','parametric equations');
plot( x,y,'b-'); hold on;
plot( X,yt,'r-');
plot( X,yn,'g-');
plot( xxx,yyy,'k*');
plot( x(1),y(1),'g*');
plot( x(N),y(N),'r*');
legend({'y vs x',['Tangent at t = ',num2str(tt)],...
    ['Normal at t = ',num2str(tt)]},...
    'location','bestoutside', 'fontsize', fsize );
grid on;
xlabel('x', 'fontsize', fsize );
ylabel('y', 'fontsize', fsize );
axis equal
title( {['x = ',x_eqa,', y = ',y_eqa], [num2str(tmin),'< t <',...
    num2str(tmax)],yt_str,yn_str}, 'fontsize', fsize  );
xlim([xmin,xmax])
ylim([ymin,ymax])
set( gca, 'fontsize', fsize );
print(gcf,['parametric ',strrep(datestr(now),':','-')],'-dpng','-r300');

%%

%F
% Function which adds . to enable elementwise matrix multiplication of
% variables in string s
function s = F(s)
s = strrep(s,'^','.^');
s = strrep(s,'*','.*');
s = strrep(s,'/','./');

%End of code