%peakfindergeneral
% Function which finds, aproximately, the coordinates of maxima and minima
% in a mathematical function y = f(x)
%
% LAST UPDATED by Andy French April 2016

function peakfindergeneral

%Number of function evaluation points
N = 1000;

%Fontsize for graphs
fsize = 16;

%

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp('  Peak finder general! Find maxima and minima ')
disp('  coordinates of a mathematical function y= f(x) ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp(' ')

%Determine inputs from user

%Define polar equation to plot
disp(' Define equation y = f(x) ')
f_eqa= input(' f(x) =  ','s');
if isempty(f_eqa)
    disp(' f(x) = x^3 -9*x^2 + 10 chosen by default.')
    f_eqa = 'x^3 -9*x^2 + 10';
end
disp(' ')

%Define range of x
disp(' Define range of x ')
xmin= input( ' xmin = ');
if isempty(xmin)
    disp(' xmin = -4 chosen by default.')
    xmin = -4;
end
xmax= input( ' xmax = ');
if isempty(xmax)
    disp(' xmax = 8 chosen by default.')
    xmax = 8;
end
disp(' ')

%Evaluate y= f(x)
x = linspace( xmin, xmax, N );
eval([ 'y = ',F(f_eqa),';']);

%Find maxima and minima coordinates
[ xs, ys, maxormin ] = peakfinder(x,y);

%Indices of stationary points which are maxima
maxima = find( maxormin == 0 );
minima = find( maxormin == 1 );

%Plot y vs x
figure('name','peakfindergeneral','color',[1 1 1])
plot( x, y, 'b');
hold on;
xlabel('x','fontsize',fsize)
ylabel('y','fontsize',fsize)
grid on;
xlimits = get(gca,'xlim');
ylimits = get(gca,'ylim');
xlim(xlimits);
ylim(ylimits);
set(gca,'fontsize',fsize)
plot( xs(maxima), ys(maxima), 'r*' );
plot( xs(minima), ys(minima), 'b*' );
xlimits = get(gca,'xlim');
ylimits = get(gca,'ylim');
plot( [0,0], ylimits, 'k-' );
plot( xlimits,[0,0], 'k-' );
for n=1:length(xs)
    text(xs(n),ys(n),['(',num2str(xs(n),2),',',...
        num2str(ys(n),2),')' ],'fontsize',fsize );
end
title( [ 'y = ',f_eqa],'fontsize',fsize )

%Print a .png file
print( gcf, ['peakfinder ',strrep(datestr(now),':','-') ],'-dpng','-r300' );

%%

%peakfinder
% Returns coordinates of maxima and minima of a vector of (x,y) values
function [ xs, ys, maxormin ] = peakfinder(x,y)

%Initialize outputs
xs = [];
ys = [];
maxormin = [];

%Step through the x,y vectors and determine maxima or minima
N = length(x);
for n=2:N-1
    
    %Test for maxima
    if ( y(n) > y(n-1) ) && ( y(n) > y(n+1) )
        xs = [ xs, x(n) ];
        ys = [ ys, y(n) ];
        maxormin = [ maxormin, 0 ];
        
        %Test for minima
    elseif ( y(n) < y(n-1) ) && ( y(n) < y(n+1) )
        xs = [ xs, x(n) ];
        ys = [ ys, y(n) ];
        maxormin = [ maxormin, 1 ];
    end
end

%%

%F
% Function which adds . to enable elementwise matrix multiplication of
% variables in string s
function s = F(s)
s = strrep(s,'^','.^');
s = strrep(s,'*','.*');
s = strrep(s,'/','./');

%End of code