%polareq
% Function which plots Cartesian (x,y) curve defined parametrically
% in terms of polar coordinates r and theta.
%
% LAST UPDATED by Andy French April 2016

function polareqa

%Fontsize for graphs
fsize = 16;

%Number of data points
N = 1000;

%

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp('  Plot curves defined as polar equations r = r(t) ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%')
disp(' ')

%Determine inputs from user

%Define polar equation to plot
disp(' Define polar equation r = r(t). Polar angle t is in radians. ')
r_eqa= F( input(' r(t) =  ','s') );
if isempty(r_eqa)
    disp(' r(t) = sqrt(t) + 2*sin(5*t) chosen by default.')
    r_eqa = 'sqrt(t) + 2*sin(5*t)';
end
disp(' ')

%Define range of polar angle t
disp(' Define range of polar angle t in radians. ')
tmin= input( ' tmin = ');
if isempty(tmin)
    disp(' tmin = 0 chosen by default.')
    tmin = 0;
end
tmax= input( ' tmax = ');
if isempty(tmax)
    disp(' tmax = 12*pi chosen by default.')
    tmax = 12*pi;
end
disp(' ')

%

% Work out vectors for t,x,y
t = linspace(tmin,tmax,N);
eval( ['r = ',r_eqa,';'] );
x = r.*cos(t);
y = r.*sin(t);

%Plot and print a .png of the parametric curve
figure('color',[1 1 1],'name','polar equations');
plot(x,y,'b-');
hold on;
plot(x(1),y(1),'g*','markersize',fsize);
plot(x(end),y(end),'r*','markersize',fsize);
grid on;
xlabel('x','fontsize',fsize);
ylabel('y','fontsize',fsize);
set(gca,'fontsize',fsize)
axis equal
title( { ['r(t) = ',r_eqa],[num2str(tmin),'< t <',num2str(tmax)] },'fontsize',fsize );
xlim( [ min([min(x),0]) , max( [max(x),0]) ] );
ylim( [ min([min(y),0]) , max( [max(y),0]) ]);
print(gcf,['polar ',strrep(datestr(now),':','-')],'-dpng','-r300');

%%

%F
% Function which adds . to enable elementwise matrix multiplication of
% variables in string s
function s = F(s)
s = strrep(s,'^','.^');
s = strrep(s,'*','.*');
s = strrep(s,'/','./');

%End of code