%polyrev
% Volume of revolution of a regular N-gon (unit radius)
%
% LAST UPDATED by Andy French April 2016

function polyrev

%Number of surface points
M = 500;

%

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Volume of revolution of a regular N-gon ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Obtain N from user
disp(' Define a positive integer N, minimum N=3 ')
N = input( ' > N = ' );
if isempty(N) || ( N < 3 ) || ( abs( N - round(N) ) > 0 )
    N = 6;
    disp(' Default N = 6 ')
end
disp(' ');

%Define vertices
theta= linspace(0,2*pi,M);
phi = linspace(0,2*pi,N+1);
[theta,phi] = meshgrid(theta,phi);
x = cos(phi);
y = sin(phi).*sin(theta);
z = sin(phi).*cos(theta);

%Plot surface
surf(x,y,z,'edgecolor','none');
interp_colormap(M);
shading interp
lighting phong
axis vis3d;
camlight
axis off
zoom(1.5);

%Print image
print( gcf, ['polyrev N=',num2str(N)],'-dpng','-r300' );

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
%
% LAST UPDATED by Andy French 14 September 2008

function new_map = interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );

%End of code