%volrev
% Plots the volume of revolution of a mathematical function, about the
% x axis and about the y axis.
%
% LAST UPDATED by Andy French April 2016

function volrev

%Surface is N^2 points
N = 500;

%Fontsize for graphs
fsize = 16;

%

%Display opening message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%')
disp('  Volumes of Revolution ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%')
disp(' ')

%Determine inputs from user

%Choose axis to rotate about
disp(' Volume of revolution about x or y axis? ')
x_or_y = input(' > ','s');
if isempty(x_or_y) || ( ( strcmp(x_or_y,'x')==0 ) && ( strcmp(x_or_y,'y')==0 ) )
    disp(' x axis chosen by default.')
    x_or_y = 'x';
end
disp(' ')

%Define function y(x) or x(y) to rotate about x or y axes
if strcmp(x_or_y,'x')
    disp(' Define function y(x) ')
    y_x = F( input(' > y(x) = ','s') );
    if isempty(y_x)
        disp(' y(x) = sin(x) chosen by default.')
        y_x = 'sin(x)';
    end
    disp(' ')
    disp(' Define x limits for volume of revolution [a,b] ')
    xlimits = input(' > [a,b] = ','s');
    if isempty(xlimits)
        disp(' [a,b] = [-2,2] chosen by default.')
        xlimits = [-2,2];
    end
    xmin = min(xlimits);
    xmax = max(xlimits);
else
    disp(' Define function x(y) ')
    x_y = F( input(' > x(y) = ','s') );
    if isempty(x_y)
        disp(' x(y) = y chosen by default.')
        x_y = 'y';
    end
    disp(' ')
    disp(' Define y limits for volume of revolution [c,d] ')
    ylimits = input(' > [c,d] = ','s');
    if isempty(ylimits)
        disp(' [c,d] = [-1,1] chosen by default.')
        ylimits = [-1,1];
    end
    ymin = min(ylimits);
    ymax = max(ylimits);
end
disp(' ')

%Display range
disp(' Define x limits of display range [a,b] ')
xlimits = input(' > [a,b] = ','s');
if isempty(xlimits)
    disp(' [a,b] = [-pi,pi] chosen by default.')
    xlimits = [-pi,pi];
end
disp(' ')
disp(' Define y limits of display range [c,d] ')
ylimits = input(' > [c,d] = ','s');
if isempty(ylimits)
    disp(' [c,d] = [-1,1] chosen by default.')
    ylimits = [-1,1];
end
disp(' ')

%

%Create figure and axis
disp(' Computing volume of revolution and creating .png file...')
figure('name','Volume of revolution',...
    'menubar', 'none','toolbar','figure')
axes('nextplot','add','units','normalized');

%Evaluate function and define radius of volume of revolution
%'cylinderettes'
theta = linspace(0,2*pi,N);
theta = repmat( theta.',[1,N]);
if strcmp(x_or_y,'x')
    x = linspace(xmin,xmax,N);
    eval( ['y = ',y_x,';'] );
    y( abs(imag(y)) > 0 ) = NaN;
    x = repmat(x,[N,1]);
    r = repmat(y,[N,1]);
    y = r.*sin(theta);
    colour = 'g';
else
    y = linspace(ymin,ymax,N);
    eval( ['x = ',x_y,';'] );
    x( abs(imag(x)) > 0 ) = NaN;
    y = repmat(y,[N,1]);
    r = repmat(x,[N,1]);
    x = r.*sin(theta);
    colour = 'b';
end
z = r.*cos(theta);

%Plot volume of revolution surface
surf(x,y,z,'facecolor',colour,'edgecolor','none');
lighting phong
alpha(0.5);
camlight

%Estimate volume and construct title string
if strcmp(x_or_y,'x')
    Vx = (xmax-xmin)*pi*sum( r(1,:).^2 )/N;
    title_str = {'Volume of revolution about x axis of';...
        ['y = ',strrep(y_x,'.','')];...
        ['Vx = ',num2str(Vx)] } ;
else
    Vy = (ymax-ymin)*pi*sum( r(1,:).^2 )/N;
    title_str = {'Volume of revolution about y axis of';...
        ['x = ',strrep(x_y,'.','')];...
        ['Vy = ',num2str(Vy)] } ;
end

%Plot y vs x (or x vs y)
if strcmp(x_or_y,'x')
    x = linspace(xlimits(1),xlimits(2),N);
    eval( ['y = ',y_x,';'] );
    y(y > ylimits(2)) = NaN;
    y(y < ylimits(1)) = NaN;
    y( abs(imag(y)) > 0 ) = NaN;
else
    y = linspace(ylimits(1),ylimits(2),N);
    eval( ['x = ',x_y,';'] );
    x(x > xlimits(2)) = NaN;
    x(x < xlimits(1)) = NaN;
    x( abs(imag(x)) > 0 ) = NaN;
end
z = zeros(1,N);
plot3(x,y,z,'k--','linewidth',2)

%Plot axis
plot3(xlimits,[0,0],[0,0],'r','linewidth',2)
plot3([0,0],ylimits,[0,0],'b','linewidth',2)

%Set axis labels and print a PNG file
axis equal
axis vis3d
xlim(xlimits)
ylim(ylimits)
xlabel('x','fontsize',fsize)
ylabel('y','fontsize',fsize)
zlabel('z','fontsize',fsize)
grid on
view([-33,40])
title(title_str,'fontsize',fsize)
set(gca,'outerposition',[0.25,0.25,0.5,0.4],'fontsize',fsize)
print(gcf,'volume of revolution','-dpng','-r300' )
disp(' ')
disp(' Volume of revolution created.')
disp(' ')

%%

%F
% Function which adds . to enable elementwise matrix multiplication of
% variables in string s
function s = F(s)
s = strrep(s,'^','.^');
s = strrep(s,'*','.*');
s = strrep(s,'/','./');

%%

%star
% Function which makes a mathematical star y = y(x)
% V is the volume of revolution.
function [y,V] = star(x, k, a)
k = abs(k);
a = abs(a);
b = 0.5*sqrt( a^2 + 4*k) - 0.5*a;
y = k./( abs(x)+b ) - b;
V = pi*k^2 * ( 1/b - 1/(a+b) ) - 2*pi*k*b*log( (a+b)/b ) + b^2 * a * pi;

%End of code