%make_cfrac
% Constructs a continued fraction from a number.
%
% LAST UPDATED by A French May 2015

function make_efrac

%Set tolerance for remainder
tol = 1e-14;

%

%Display opening message to command window
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Creates a continued fraction representation of a number ' )
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ')

%Get inputs from user
disp(' Choose from a default list or make your own ')
disp(' ')
disp(' 1 - Golden ratio ')
disp(' 2 - pi/2 ')
disp(' 3 - e ' )
disp(' 4 - sqrt(2)' )
disp(' 5 - sqrt(42)' )
disp(' 6 - Define your own number')
disp(' ')
choice = input(' Choice = ');
format long
if isempty(choice)
    disp(' Default option is #1, the Golden ratio ')
    choice = 1;
end
if choice==1
    %Golden ratio
    x = (1+sqrt(5))/2;
    disp([' The golden ratio (1+sqrt(5))/2 = ',num2str( (1+sqrt(5))/2, 15 )] );
elseif choice==2
    %pi/2
    x = pi/2;
    disp([' pi/2 = ',num2str( pi/2, 15 )] );
elseif choice==3
    %e
    x = exp(1);
    disp([' e = ',num2str( exp(1), 15 ) ] );
elseif choice==4
    %sqrt(2)
    x = sqrt(2);
    disp([' Square root of 2 = ',num2str( sqrt(2), 15 )] );
elseif choice==5
    %sqrt(42)
    x = sqrt(42);
    disp([' Square root of 42 = ',num2str( sqrt(42), 15 ) ] );
elseif choice==6
    x = input(' Number = ');
    if isempty(x)
        disp(' Default option is for the Golden ratio')
        disp([' x = ',num2str( (1+sqrt(5))/2, 15 ) ] );
        x = (1+sqrt(5))/2;
    end
end

%Get from user the number of nested fraction levels
disp(' ')
disp(' Set the number of continued fraction levels ... ')
N = input(' N = ');
if isempty(N);
    disp(' Default N is 20 ' );
    N = 20;
end
disp(' ');


%Initialize output and initial remainder
y = floor(x);
r = x - floor(x);

%Iterate in a loop until either the remainder r is zero, or the number of
%loops equates to N. Determine integers of continued fraction and store
%these in vector y
n=1;
while ( abs(r)> tol ) && ( n <= N )
    y = [y,floor(1/r)];
    r = 1/r - floor(1/r);
    n = n + 1;
end

%Display result to command window
disp([' Number vector = ',vec2str(y) ] );
disp(' ');

%Work backwards through the fraction and hence evaluate it
x = y(end);
for n=1:length(y)-1
    x = y( end - n ) + 1/x;
end
disp([' Continued fraction = ',num2str(x, 15)])
disp(' ')

%%

%Turn a vector of numbers x into a string s
function s = vec2str(x)
s = '[';
for n=1:numel(x)
    s = [s,num2str(x(n)),','];
end
%Remove last comma and replace with a ']'
s(end) = '';
s = [s,']'];

%End of code
