%numberpatterns
% Function which generates an NxN grid of integers and counts and
% highlights the elements which conform to a given rule. A separate rule
% defines how the (m,n) th element is formed from integers m and n.
%
% LAST UPDATED by Andy French April 2016

function numberpatterns

%%% Hard coded inputs %%%

%Set font size
fsize = 8;

%Set whether numbers are printed (use this for large N)
display_numbers = 1;

%Shade numbers by size that don't obey the rule
shade_by_size = 1;

%Set maximum N for chequerboard plot, grid, numbers
N_chequer = 1000;
N_grid = 150;
N_text = 22;

%

%Introduction message
disp(' ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%' )
disp('  Number pattern generator ')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%' )
disp(' ')

%Size of (square) grid 1:N in rows and columns
disp(' Define size of (square) grid 1:N in rows and columns ')
N = input( ' > N = ');
if isempty(N)
    N = 20; 
    disp(' Default N = 20'); 
end
disp(' ');

%Define grid function
disp(' Define grid generating function')
disp(' ')
disp(' 1    Multiplication')
disp(' 2    Addition')
disp(' 3    Sequential integers')
disp(' 4    Subtraction')
disp(' 5    Division')
disp(' 6    Power')
disp(' ')
g = input(' Option number = ');
if g==1
    gridfunc = 'multiply';
elseif g==2
    gridfunc = 'add';
elseif g==3
    gridfunc = 'sequential integers';
elseif g==4
    gridfunc = 'subtract';
elseif g==5
    gridfunc = 'divide';
elseif g==6
    gridfunc = 'power';
else
    gridfunc = 'multiply';
end
disp(' ')
disp([' Selected grid function is: ',gridfunc]);

%Define highligting and counting rule involving output of gridfunc, y and
%also n,m
disp(' ');
disp(' Define highlighting and counting rule')
disp(' ')
disp(' 1    Primes')
disp(' 2    Squares')
disp(' 3    Cubes')
disp(' 4    Odd')
disp(' 5    Even')
disp(' 6    More than k')
disp(' 7    More or equal to k')
disp(' 8    Less than k')
disp(' 9    Less than or equal to k')
disp(' ')
g = input(' Rule number = ');
if g==1
    rule = 'prime';
elseif g==2
    rule = 'square';
elseif g==3
    rule = 'cube';
elseif g==4
    rule = 'odd';
elseif g==5
    rule = 'even';
elseif g==6
    rule = 'more than k';
elseif g==7
    rule = 'more or equal to k';
elseif g==8
    rule = 'less than k';
elseif g==9
    rule = 'less than or equal to k';
else
    rule = 'square';
end
disp(' ')
disp([' Selected rule is: ',rule]);

%Constant for k rule
if g>=6
    disp(' ')
    disp(' Choose constant k ')
    k = input(' > k = ');
    if isempty(k)==1; 
        k = 300; 
        disp(' Default k = 300.')
    end
else
    k=0;
end

%

%Generate grid.
disp(' ')
t0 = tic;
disp(' * Generating grid. Please be patient!')
n = 1:N;
m = 1:N;
[m,n] = meshgrid(m,n);
y = F( m, n, gridfunc);
miny = min(min(y));
maxy = max(max(y));

%Generate binary matrix determining whether rule has been passed.
[p,num_pass] = R( y, m, n, k, rule);

%Plot grid
figure('name','Number patterns','color',[1 1 1],'renderer','opengl')
axes('nextplot','add','ticklength',[0,0],'xtick',[],'ytick',[]);
caxis( [miny,maxy] );
interp_colormap(500);

%Plot shaded chequerboard, unless N >= 1000
if N<N_chequer
    
    %Numbers which don't pass the rule
    if shade_by_size == 1
        image((1:N) + 0.5,(1:N) + 0.5,y,'alphadata',0.3*ones(N,N),'CDataMapping','scaled');
    end
    
    %Numbers which pass the rule
    image((1:N) + 0.5,(1:N) + 0.5,y,'alphadata',double(p),'CDataMapping','scaled');
    
    %Create grid
    if N<N_grid
        for q = 1:N+1
            %y grid line
            plot([q,q],[1,N+1],'k');
        end
        %x grid line
        for r = 1:N+1
            plot([1,N+1],[r,r],'k');
        end
    end
else
    %Plot blank spot at top right of axes to set the size
    plot(N,N,'w.');
end

%Plot dots at (m,n) values where rule is passed, or plot the numbers
%themselves if N is small enough and are specified by the inout flag
if N>N_text
    i = find(p==1);
    xx = zeros(size(i));
    yy = zeros(size(i));
    xx(:) = m(i);
    yy(:) = n(i);
    plot( xx+0.5, yy+0.5, 'b.','markersize',3)
else
    if display_numbers == 1;
        %Set axis tick marks
        set(gca,'xtick',(1:N) + 0.5,'xticklabel',1:N,'ytick',(1:N) +0.5,'yticklabel',1:N);
        
        %Position outputs inside the square
        for mm=1:N
            for nn=1:N
                t=text( m(mm,nn)+0.5, n(mm,nn)+0.5, 1, num2str( y(mm,nn) ),...
                    'fontsize',fsize,'HorizontalAlignment','center','color',[0 0 0],'fontweight','bold' );
                if p(mm,nn)==1
                    set(t,'color',[1,1,1]);
                end
            end
        end
    end
end
axis equal;
axis tight

%Plot square or cube lines if the option is set
if strcmp(rule,'square')==1
    square_lines(gca,N);
elseif strcmp(rule,'cube')==1
    Npower_lines(3, gca, N);
end

%Set title string with number of elements that pass the rule underneath
%defined text
if g>=6
    rule = strrep( rule, 'k',num2str(k) );
end
title_str = {
    [gridfunc,' grid for 1 : ',num2str(N),...
    '. Rule = ',rule,'.'],...
    ['Number of elements obeying rule = ',...
    num2str(num_pass),' (',num2str(100*num_pass/(N^2)),'%)']
    };
title(title_str);
set(gca,'units','normalized','position',[0.15,0.1,0.75,0.75] );

%Save 300dpi PNG file of grid
filename = ['Number patterns -',gridfunc,' rule=',rule,' N=',num2str(N),...
    ' ',strrep(datestr(now),':','-'),'.png'];
print( gcf, '-dpng','-r300',filename)
close(gcf);

%Display ending message
disp('')
disp(' * Number pattern grid generated and written to file:');
disp(['   Number patterns -',gridfunc,' rule=',rule,' N=',num2str(N),...
    ' ',strrep(datestr(now),':','-'),'.png']);
disp(['   in ',num2str(toc(t0)),' seconds.'])
disp(' ')

%%

%Grid function of integers n and m
function y = F( m, n, gridfunc)
if strcmp( gridfunc, 'add' ) == 1
    y = n + m;
elseif strcmp( gridfunc, 'subtract') == 1
    y = n - m;
elseif strcmp( gridfunc, 'multiply') == 1
    y = m.*n;
elseif strcmp( gridfunc, 'divide') == 1
    y = m./n;
elseif strcmp( gridfunc, 'power') == 1
    y = m.^n;
elseif strcmp( gridfunc, 'sequential integers') ==1
    dim = size(m);
    N = dim(1);
    y = m + (n-1)*N;
else
    %Default is the multiplication grid
    y = m.*n;
end

%%

%Rule function
% Selection rule. Matrix of binary outputs given grid function output y
% Row integers m, Column integers n, constant k and rule
% string rule.
function [p,num_ones] = R( y, m, n, k, rule)
tol = 10^-10;
p = zeros(size(y));
if strcmp( rule, 'prime') == 1;
    p = isprime(y);
elseif strcmp( rule, 'square') == 1;
    p = abs( sqrt(y) - round( sqrt(y) ) ) < tol;
elseif strcmp( rule, 'cube') == 1;
    p = abs( y.^(1/3) - round( y.^(1/3) ) ) < tol;
elseif strcmp( rule, 'odd') == 1;
    p = abs( y/2 - round( y/2 ) ) > tol;
elseif strcmp( rule, 'even') == 1;
    p = abs( y/2 - round( y/2 ) ) < tol;
elseif strcmp( rule, 'more than k') == 1
    p = y>k;
elseif strcmp( rule, 'more or equal to k') == 1
    p = y>=k;
elseif strcmp( rule, 'less than k') == 1
    p = y<k;
elseif strcmp( rule, 'less than or equal to k') == 1
    p = y<=k;
end

%Count the number of ones
num_ones = sum(sum(p));

%%

% Square lines function
% This function plots y(x) lines of the form y = k*x in an integer
% multiplication square x,y where x and y are positive integers. At each
% point (x,y) in the square the value z = x*y. Square numbers are when
% z^1/2 = integer.
function square_lines(ax,xmax)

%% Inputs %%

%Define number of x and y values
num = 1000;

%

%Create axes if ax is empty
if isempty(ax)==1
    figure('name','square lines','color',[1 1 1])
    ax = axes;
    xlabel('x');
    ylabel('y');
    title('Square numbers in a multiplication grid')
end

%Construct vector of gradients
k = [1/16,1/9,1/4,4/9,9/16,1,9/4,4,9,16];

%Plot lines
axes(ax);
hold on;
for n=1:numel(k)
    plot( 1:xmax, k(n)*(1:xmax), 'r','linewidth',0.1 );
end

%%

% Cube lines function
% This function plots y(x) lines of the form y = k*x^n in an integer
% multiplication square x,y where x and y are positive integers. At each
% point (x,y) in the square the value z = x*y. Cube numbers are when
% z^1/3 = integer.
function cube_lines(ax,xmax)

%%

% N-power lines function
% This function plots y(x) lines of the form y = k*x^m in an integer
% multiplication square x,y where x and y are positive integers. At each
% point (x,y) in the square the value z = x*y. N-power numbers are when
% z^1/N = integer.
function Npower_lines(N, ax,xmax)

%% Inputs %%

%Define number of x and y values
num = 1000;

%Define extent of integer parameters
M = 10;

%Create axes if ax is empty
if isempty(ax)==1
    figure('name','square lines','color',[1 1 1])
    ax = axes;
    xlabel('x');
    ylabel('y');
    title('Square numbers in a multiplication grid')
end

%Plot lines
axes(ax);
hold on;
x = 1:xmax;
for n=1:M
    for Q=1:M
        for R=1:1
            for S=1:1
                m = log10( (n^N)/(Q*S) ) / log10(Q/R)
                k = (n^N)/( Q^(m+1) )
                plot( x, k*(x.^m), 'r','linewidth',0.1 );
            end
        end
    end
end

%%

%interp_colormap
% Function which interpolates current colourmap to yield better graduated
% shading. N is number of possible colours.
function interp_colormap(N)

%Get current colourmap
map = colormap;

%Initialise new colormap
new_map = ones(N,3);

%Get size of current colormap and initalise red,green,blue vectors
dim = size(map);
R = ones(1,dim(1));
G = ones(1,dim(1));
B = ones(1,dim(1));
RR = ones(1,N);
GG = ones(1,N);
BB = ones(1,N);

%Populate these with current colormap
R(:) = map(:,1);
G(:) = map(:,2);
B(:) = map(:,3);

%Interpolate to yield new colour map
x = linspace( 1, dim(1), N );
RR = interp1( 1:dim(1), R, x );
GG = interp1( 1:dim(1), G, x );
BB = interp1( 1:dim(1), B, x );
new_map(:,1) = RR(:);
new_map(:,2) = GG(:);
new_map(:,3) = BB(:);

%Set colormap to be new map
colormap( new_map );


%End of code