%multivalue_trig
% Function which finds multiple solutions of trignometric equations of the
% form t( f(x) ) = k where t is sin, cos or tan.
% NOTE f(x) must not have any multiple values over the range of x chosen.
%
% LAST UPDATED by Andy French April 2016

function multivalue_trig

%Fontsize
fsize = 18;

%Number of data points
N = 1000;

%

%Display greeting message
disp(' ');
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%' )
disp('  MULTI-VALUE TRIGONOMETRY')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%%' )
disp(' ' )

%Get user inputs

%Define trig function
disp(' Define trig function sin cos or tan')
t = input( ' > ', 's' );
if ( strcmp(t,'sin')==1 ) || ( strcmp(t,'cos')==1 ) || ( strcmp(t,'tan')==1 )
else
    t = 'sin';
    disp(' Default trig function chosen as sin.')
    disp(' ')
end

%Define k
disp(' ');
disp(' Define k s.t. trig function = k is the equation to solve ')
k = input(' > k = ');
if isempty(k)
    k = 0.5;
    disp(' Default k = 0.5 ')
end

%Horizontal axis in degrees?
disp(' ');
disp( ' Display horizontal axis in degrees? (Y/N)' )
deg_display = input(' > ');
if isempty( deg_display )
    deg_display = 'Y';
    disp(' Degrees, not radians, will be used.' )
end

%Define argument of trig function t( f(x) ) IN RADIANS
disp(' ');
fx = input(' Define argument of trig function t( f(x) ) IN RADIANS \n > f(x) = ','s');
if isempty( fx )
    fx = 'x';
    disp(' f(x) = x set as default. ' )
end

%Define inverse function of argument of trig function t( f(x) );
disp(' ');
gx = input(' Define inverse function of argument of trig function t( f(x) ) \n > g(x) = ','s');
if isempty( gx )
    gx = 'x';
    disp(' Inverse function g(x) = x set as default. ' )
end

%Define range IN RADIANS of allowed solutions
disp(' ');
xok = input('Define range IN RADIANS of allowed solutions \n > [a,b] = ');
if isempty( xok )
    xok = [0,2*pi];
    disp(' [a,b] = [0,2*pi]' )
end

%

%Get solutions and vectors of x,y,f
[x,xx,y,f] = multi_value_trig( t, fx, gx, k, xok, N );

%Print solutions in degrees
disp(' ')
disp([' Solutions to y = ',t,'( ',fx,' ) = ',num2str(k),...
    ' in range [',num2str(180*xok(1)/pi),',',num2str(180*xok(2)/pi),'] are...'] )
for n=1:length(xx)
    disp( [ ' ',num2str(180*xx(n)/pi), ' deg = ',num2str(xx(n)),' rad'] );
end
disp(' ');

%Plot trig function vs x
figure('color',[1 1 1])
if strcmp( deg_display, 'Y') ==1
    x = x*180/pi;
    xx = xx*180/pi;
end
plot( x, y, 'b');
hold on;
plot( [x(1),x(end)],[k,k] ,'r');
ylimit = get(gca,'ylim');
plot( xx,k*ones(1,length(xx)),'r*')
if strcmp( deg_display, 'Y') ==1
    xlabel('x /degrees','fontsize',fsize)
else
    xlabel('x /radians','fontsize',fsize)
end
ylabel(['y = ',t,'( ',strrep(fx,'^','\^'),' )'],'fontsize',fsize)
title(['y = ',t,'( ',strrep(fx,'^','\^'),' )'],'fontsize',fsize)
set(gca,'fontsize',fsize)
axis tight
grid on
ylimit = [max( [min(y),-max(x)] ), min( [max(y),max(x)] ) ];
set(gca,'ylim',ylimit)
set(gca,'xlim',[min(x),max(x)])
print( gcf, '-r300', '-dpng','multi_valued_trig' )

%%

%multi_value_trig
% Function which finds multiple solutions of trignometric equations of the
% form t( f(x) ) = k where t is sin, cos or tan.
% NOTE f(x) must not have any multiple values over the range of x chosen.
%
% LAST UPDATED by Andy French Sept 2012
%
% Syntax: [x,xx,y,f] = multi_value_trig( t, fx, gx, k, xok, N )
%
% t       Trig function: either 'sin', 'cos' or 'tan'
% fx      Argument of trig function t( f(x) ) IN RADIANS
% gx      Inverse function of argument of trig function t( f(x) );
% k       RHS of equation to be solved t( f(x) ) = k
% xok     Define range IN RADIANS of allowed solutions
% N       Number of data points
%
% x       Allowed range of x, N linearly spaced points
% xx      Vector of solutions in radians
% y       y = t( f(x) )
% f       f(x)

function [x,xx,y,f] = multi_value_trig( t, fx, gx, k, xok, N )

%Define range of x and fx
x = linspace( xok(1), xok(2), N);
f = F(x,fx);

%Find 'basic' solutions and then compute multiple values

%Work out deviations from 0, 270, 90 or -90
if strcmp( t,'sin' )
    y = sin(f);
    b = asin(k);
    df = 2*pi;
    if abs(k) > 1
        b = NaN;  %i.e. ignore complex solutions
        f0=NaN;
    elseif k<0
        b = 90*pi/180 - abs(b);
        f0 = -pi/2;
    else
        b = 90*pi/180 - b;
        f0 = pi/2;
    end
elseif strcmp( t,'cos' )
    y = cos(f);
    b = acos(k);
    df = 2*pi;
    if abs(k) > 1
        b = NaN;  %i.e. ignore complex solutions
        f0 = NaN;
    elseif k<0
        b = pi - abs(b);
        f0 = pi;
    else
        f0 = 0;
    end
elseif strcmp( t,'tan' )
    y = tan(f);
    b = atan2(k,1);
    df = pi;
    f0=0;
end

%Compute multiple values and add these to array f
f = f0+b;
if strcmp( t,'tan' )==0
    f = [f,f0-b];
end
n = 1;
ff = f0+b;
while ( ff> F(xok(1),fx) )
    if strcmp( t,'tan' )==0
        ff = f0 -b - n*df;
        f = [ff,f];
    end
    ff = f0 +b - n*df;
    f = [ff,f];
    n = n + 1;
end
n=1;
ff = f0+b;
while  ( ff < F(xok(2),fx) );
    if strcmp( t,'tan' )==0
        ff = f0 -b + n*df;
        f = [f,ff];
    end
    ff = f0 +b + n*df;
    f = [f,ff];
    n = n + 1;
end
f( f < F(xok(1),fx) ) = [];
f( f > F(xok(2),fx) ) = [];

%Convert solutions to x not f(x)
xx = sort( F(f,gx) );

%%

%Evaluate function defined as a string
function y = F(x,fx)
fx = strrep(fx,'^','.^');
fx = strrep(fx,'/','./');
fx = strrep(fx,'*','.*');
eval(['y=',fx,';'])
y = real(y);

%End of code