%Function which compares different trignometric functions
%
% LAST UPDATED by Andy French November 2013.

function trig_compare

%Define number of data points
N = 200;

%Fontsize
fsize = 16;

%Define trig functions (of angle x) to compare
trig_funcs = {
    'sin(x)',...
    'cos(x)',...
    'tan(x)'
    };

%Set radians or degrees
angle_unit = 'degrees';
% angle_unit = 'radians';

%Define range of angle (based on unit above)
% angle_range = [0,4*pi];
angle_range = [0,720];

%Define angle ticks (set as [] for auto)
% angle_ticks = 0 : 1 : 4*pi;
angle_ticks = 0 : 90 : 720;

%Set y limits (set as [] for auto)
ylimits = [-2,2];

%

%Determine title string
title_str = [];
for n=1:length(trig_funcs)
    title_str = [title_str,trig_funcs{n},' vs '];
end
title_str = title_str(1:end-4);

%Create array of angles in radians
x = linspace( angle_range(1),angle_range(2),N);
if strcmp( angle_unit,'degrees' )==1
    x = (pi/180)*x;
end
xx = repmat(x.',[1,length(trig_funcs)]);

%Evaluate trig functions and store in an array
y = [];
for n=1:length(trig_funcs)
    eval([ 'yy = ',strrep( trig_funcs{n},'tan','new_tan'),';']);
    y = [y,yy.'];
end

%Convert angle range to degress, if option is set
if strcmp( angle_unit,'degrees' )==1
    xx = 180*xx/pi;
    x = 180*x/pi;
end

%Create figure
figure('color',[1 1 1],'name','Trig compare',...
    'units','normalized','position',[0.2594    0.2880    0.5120    0.3926])

%Plot trig function vs theta with solutions labelled
axes('Nextplot','add');
plot(xx,y);
hold on;
if strcmp( angle_unit,'degrees' )==1
    xlabel('\theta /degrees','fontsize',fsize)
else
    xlabel('\theta /radians','fontsize',fsize)
end
ylabel('y','fontsize',fsize)
title( title_str,'fontsize',fsize)
set(gca,'fontsize',fsize)
grid on
if ~isempty(ylimits)
    set(gca,'ylim',ylimits);
else
    ylimits = get(gca,'ylim');
end
set(gca,'xlim',[min(x),max(x)])
plot([min(x),max(x)],[0,0],'k')
plot([0,0],ylimits,'k')
if ~isempty(angle_ticks)
    set( gca, 'xtick',angle_ticks)
end
box on
legend( trig_funcs,'location','bestoutside')

%Print a PNG of the figure
print( gcf, '-r300', '-dpng','trig_func compare.png' )

%%

%Modify tan function to yield NaN for x close to pi/2 + N*pi
function y = new_tan(x)
y = tan(x);
y( abs(y) > 5 ) = NaN;

%End of code