%unit_circle_trig
% This function will display solutions (a) of t(theta) = k
% on the unit circle, and also on a graph of the trignometric function t
% t can be 'sin', 'cos' or 'tan'.
%
% LAST UPDATED by Andy French April 2016

function unit_circle_trig

%Define number of data points
N = 200;

%Fontsize
fsize = 18;

%

%Display greeting message
disp(' ');
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp('  Unit Circle Trigonometry')
disp(' %%%%%%%%%%%%%%%%%%%%%%%%%% ')
disp(' ' )

%Get user inputs

%Define trig function
disp(' Define trig function sin cos or tan')
trig_func = input( ' > ', 's' );
if ( strcmp(trig_func,'sin')==1 ) || ( strcmp(trig_func,'cos')==1 ) || ( strcmp(trig_func,'tan')==1 )
else
    trig_func = 'sin';
    disp(' Default trig function chosen as sin.')
    disp(' ')
end

%Define k
disp(' ');
disp(' Define k s.t. trig function = k is the equation to solve ')
k = input(' > k = ');
if isempty(k)
    k = 0.5;
    disp(' Default k = 0.5 ')
end

%Horizontal axis in degrees?
disp(' ');
disp( ' Display horizontal axis in degrees? (Y/N)' )
deg_display = input(' > ');
if isempty( deg_display )
    deg_display = 'Y';
    disp(' Degrees, not radians, will be used.' )
end

%

%Get solutions to equation t(a) = k
xok = [0,2*pi];
[theta,a,t,f] = multi_value_trig( trig_func, 'x', 'x', k, xok, N );

%Create unit cirle
x = cos(theta);
y = sin(theta);

%Print solutions in degrees
disp(' ')
disp([' Solutions to ',trig_func,'(theta) = ',num2str(k),...
    ' in range [',num2str(180*xok(1)/pi),',',num2str(180*xok(2)/pi),'] are ...'] )
for n=1:length(a)
    disp( [ ' ',num2str(180*a(n)/pi), ' deg = ',num2str(a(n)),' rad'] );
end
disp(' ');

%Create figure
figure('color',[1 1 1],'name','Unit circle trig',...
    'units','normalized','position',[0.2594    0.2880    0.5120    0.3926])

%Plot unit circle diagram
subplot(1,2,1)
plot(x,y,'b')
hold on;
for n=1:length(a)
plot([0,cos(a(n))],[0,sin(a(n))],'r');
plot([0,cos(a(n))],[0,sin(a(n))],'r*');
end
axis equal
xlim([-1,1]);
ylim([-1,1]);
grid on
xlabel('x = cos(\theta)','fontsize',fsize)
ylabel('y = sin(\theta)','fontsize',fsize)
title(['Unit circle: ',trig_func,'(\theta) = ',num2str(k)],'fontsize',fsize)
plot([-1,1],[0,0],'k')
plot([0,0],[-1,1],'k')
set(gca,'fontsize',fsize)

%Plot trig function vs theta with solutions labelled
subplot(1,2,2)
if strcmp( deg_display, 'Y') ==1
    theta = theta*180/pi;
    a = a*180/pi;
end
plot( theta, t, 'b');
hold on;
plot( [theta(1),theta(end)],[k,k] ,'r');
plot( a,k*ones(1,length(a)),'r*')
if strcmp( deg_display, 'Y') ==1
    xlabel('\theta /degrees','fontsize',fsize)
    theta_ticks = [0,90,180,270,360];
else
    xlabel('\theta /radians','fontsize',fsize)
    theta_ticks = [0,pi/2,pi,3*pi/2,2*pi];
end
if strcmp(trig_func,'tan')==1
    ylimit = [min(-1.1*k,-10),max(1.1*k,10)];
else
    ylimit = [min(-1.1*k,-1),max(1.1*k,1)];
end
set(gca,'xtick',theta_ticks);
ylabel([trig_func,'(\theta)'],'fontsize',fsize)
title([trig_func,'(\theta) = ',num2str(k)],'fontsize',fsize)
set(gca,'fontsize',fsize)
grid on
set(gca,'ylim',ylimit)
set(gca,'xlim',[min(theta),max(theta)])
plot([min(theta),max(theta)],[0,0],'k')
plot([0,0],ylimit,'k')

%Print a PNG of the figure
print( gcf, '-r300', '-dpng',[trig_func,'=',num2str(k),'.png'] )

%%

%multi_value_trig
% Function which finds multiple solutions of trignometric equations of the
% form t( f(x) ) = k where t is sin, cos or tan.
% NOTE f(x) must not have any multiple values over the range of x chosen.
%
% LAST UPDATED by Andy French Sept 2012
%
% Syntax: [x,xx,y,f] = multi_value_trig( t, fx, gx, k, xok, N )
%
% t       Trig function: either 'sin', 'cos' or 'tan'
% fx      Argument of trig function t( f(x) ) IN RADIANS
% gx      Inverse function of argument of trig function t( f(x) );
% k       RHS of equation to be solved t( f(x) ) = k
% xok     Define range IN RADIANS of allowed solutions
% N       Number of data points
%
% x       Allowed range of x, N linearly spaced points
% xx      Vector of solutions in radians
% y       y = t( f(x) )
% f       f(x)

function [x,xx,y,f] = multi_value_trig( t, fx, gx, k, xok, N )

%Define range of x and fx
x = linspace( xok(1), xok(2), N);
f = F(x,fx);

%Find 'basic' solutions and then compute multiple values

%Work out deviations from 0, 270, 90 or -90
if strcmp( t,'sin' )
    y = sin(f);
    b = asin(k);
    df = 2*pi;
    if abs(k) > 1
        b = NaN;  %i.e. ignore complex solutions
        f0=NaN;
    elseif k<0
        b = 90*pi/180 - abs(b);
        f0 = -pi/2;
    else
        b = 90*pi/180 - b;
        f0 = pi/2;
    end
elseif strcmp( t,'cos' )
    y = cos(f);
    b = acos(k);
    df = 2*pi;
    if abs(k) > 1
        b = NaN;  %i.e. ignore complex solutions
        f0 = NaN;
    elseif k<0
        b = pi - abs(b);
        f0 = pi;
    else
        f0 = 0;
    end
elseif strcmp( t,'tan' )
    y = tan(f);
    b = atan2(k,1);
    df = pi;
    f0=0;
end

%Compute multiple values and add these to array f
f = f0+b;
if strcmp( t,'tan' )==0
    f = [f,f0-b];
end
n = 1;
ff = f0+b;
while ( ff> F(xok(1),fx) )
    if strcmp( t,'tan' )==0
        ff = f0 -b - n*df;
        f = [ff,f];
    end
    ff = f0 +b - n*df;
    f = [ff,f];
    n = n + 1;
end
n=1;
ff = f0+b;
while  ( ff < F(xok(2),fx) );
    if strcmp( t,'tan' )==0
        ff = f0 -b + n*df;
        f = [f,ff];
    end
    ff = f0 +b + n*df;
    f = [f,ff];
    n = n + 1;
end
f( f < F(xok(1),fx) ) = [];
f( f > F(xok(2),fx) ) = [];

%Convert solutions to x not f(x)
xx = sort( F(f,gx) );

%%

%Evaluate function defined as a string
function y = F(x,fx)
fx = strrep(fx,'^','.^');
fx = strrep(fx,'/','./');
fx = strrep(fx,'*','.*');
eval(['y=',fx,';'])
y = real(y);

%End of code